@@ -1,7 +1,7 @@
 #*********************************************************************
 #*** ResourcePool::Resource
 #*** Copyright (c) 2002 by Markus Winand <mws@fatalmind.com>
-#*** $Id: Resource.pm,v 1.19 2002/10/06 13:43:21 mws Exp $
+#*** $Id: Resource.pm,v 1.23 2002/10/12 17:25:00 mws Exp $
 #*********************************************************************
 
 package ResourcePool::Resource;
@@ -9,13 +9,14 @@ package ResourcePool::Resource;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = "0.9909";
+$VERSION = "0.9910";
 
 sub new($@) {
         my $proto = shift;
         my $class = ref($proto) || $proto;
         my $self = {};
-	$self->{PlainResource} = {shift => $self};
+	$self->{PR} = $self;
+	$self->{ARGUMENT} = shift;
 	$self->{VALID} = 1;
 
         bless($self, $class);
@@ -24,36 +25,30 @@ sub new($@) {
 }
 
 sub close($) {
-	my ($self) = @_;
 	return undef;
 }
 
 sub fail_close($) {
-	my ($self) = @_;
 	warn "ResourcePool::Resource: closing failed Resource\n";
 	return undef;
 }
 
 sub precheck($) {
-	my ($self) = @_;
-	return $self->{VALID};
+	return $_[0]->{VALID};
 }
 
 sub postcheck($) {
-	my ($self) = @_;
-	return $self->{VALID};
+	return $_[0]->{VALID};
 }
 
 sub get_plain_resource($) {
-	my ($self) = @_;
-	return $self->{PlainResource};
+	return $_[0]->{PR};
 }
 
 ### ### Private part starts here
 
 sub _my_very_private_and_secret_test_hook($$) {
-	my ($self, $valid) = $_;
-	$self->{VALID} = $valid;
+	$_[0]->{VALID} = $_[1];
 }
 
 1;
